#!perl

use strict;
use warnings;

use Test::More;
use Test::Fatal;

BEGIN {
    use_ok('MOP::Role');
}

=pod

TODO:
- test method conflicts

=cut

diag 'TODO - test method conflicts';

done_testing;
